package com.nd.smartcan.content.s3.upload;

import java.io.File;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Map;

/**
 * Created by Administrator on 2016/3/28.
 */
public interface IUpLoader {

    public static final String UPLOAD_HOST = "Host";
    public static final String UPLOAD_URL = "Url";
    public static final String UPLOAD_DATE = "Date";
    public static final String UPLOAD_AUTHORIZATION = "Authorization";
    public static final String UPLOAD_NAME = "name";

    /**
     * Cephs3旁路上传，获取Cephs3上传的token。
     * @param type 文件上传类型 0-一次性上传;1-初始化分块上传;2-分块上传第N块文件;3-完成分块上传
     * @return  json字符串
     */
    String getTokenParams(int type);

    /**
     * Cephs3旁路上传,一次性上传文件
     * @param uploadData
     * uploadData：{
      “upload_host”：第三方平台上传文件的域名
      “upload_token”：第三方平台上传的Token
      “upload_bucket”：第三方平台上传的Bucket名称
      “upload_platform”：第三方平台上传的平台名称
      “upload_date”：此次操作的时间(服务端时间),且必须为 HTTP1.1 中支持的 GMT 格式
      “upload_url”:第三方平台上传完整URL
     }
     * @param file:需要上传的文件
     * @return
     */
    String putObject(Map<String,Object> uploadData,File file);

    /**
     * Cephs3旁路上传,一次性上传文件
     * @param uploadData
     * uploadData：{
      “upload_host”：第三方平台上传文件的域名
      “upload_token”：第三方平台上传的Token
      “upload_bucket”：第三方平台上传的Bucket名称
      “upload_platform”：第三方平台上传的平台名称
      “upload_date”：此次操作的时间(服务端时间),且必须为 HTTP1.1 中支持的 GMT 格式
      “upload_url”:第三方平台上传完整URL
      }
     * @param inputStream:需要上传的文件流
     * @param length：需要上传的文件长度
     * @return
     */
    String putObject(Map<String,Object> uploadData,InputStream inputStream ,Long length);

    /**
     * Cephs3旁路上传,分块上传 - 初始化分块上传文件
     * @param mParamsMap
     * @return
     */
    String initiateMultipartUpload(Map<String,Object> mParamsMap);

    /**
     * Cephs3旁路上传,分块上传 - 分块上传第N块文件
     * @param uploadData
     * @return
     */
    String multipartUploadPartByFile(Map<String,Object> uploadData,String uploadId,File file);

    /**
     * Cephs3旁路上传,分块上传 - 分块上传第N块文件
     * @param uploadData
     * @return
     */
    String multipartUploadPartByStream(Map<String,Object> uploadData,String uploadId,InputStream inputStream,Long length);

    /**
     * Cephs3旁路上传,分块上传 - 完成分块上传文件
     * @param mParamsMap
     * @return
     */
    void completeMultipartUpload(Map<String,Object> mParamsMap,String uploadId,ArrayList<String> etags) throws Exception;

}
