package com.nd.smartcan.content.s3.upload;

import android.util.Log;
import com.nd.android.smartcan.network.NetworkOptions;
import com.nd.android.smartcan.network.NetworkRequest;
import com.nd.android.smartcan.network.base.ByteArrayPool;
import com.nd.android.smartcan.network.base.PoolingByteArrayOutputStream;
import com.nd.android.smartcan.network.exception.*;
import com.nd.android.smartcan.network.mime.TypedInput;
import com.nd.android.smartcan.network.mime.TypedOutput;
import com.nd.android.smartcan.network.retry.RetryPolicy;
import com.nd.android.smartcan.network.util.HttpsConnectionHelper;
import com.nd.smartcan.commons.util.logger.Logger;
import org.apache.http.*;
import org.apache.http.conn.ConnectTimeoutException;
import org.apache.http.entity.BasicHttpEntity;
import org.apache.http.message.BasicHeader;
import org.apache.http.message.BasicHttpResponse;
import org.apache.http.message.BasicStatusLine;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLSocketFactory;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.SocketTimeoutException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2016/3/30.
 */
public class S3NetworkClientImpl {
    //private static final String HEADER_CONTENT_TYPE = "Content-Type";
    //private static final String TAG = "NetworkClient";
    //private static final int DEFAULT_POOL_SIZE = 4096;
    //private static final int DEFAULT_MAX_NUM_408_RETRIES = 1;
    private final SSLSocketFactory mSslSocketFactory;
    private final HostnameVerifier mHostnameVerifier;
    protected final ByteArrayPool mPool;
    private int retries408;

    public S3NetworkClientImpl() {
        this(HttpsConnectionHelper.getAllHostsValidSocketFactory(), (HostnameVerifier)null);
    }

    public S3NetworkClientImpl(SSLSocketFactory sslSocketFactory) {
        this(sslSocketFactory, HttpsConnectionHelper.getAllHostsValidVerifier());
    }

    public S3NetworkClientImpl(SSLSocketFactory sslSocketFactory, HostnameVerifier hostnameVerifier) {
        this.retries408 = 0;
        this.mHostnameVerifier = hostnameVerifier;
        this.mSslSocketFactory = sslSocketFactory;
        this.mPool = new ByteArrayPool(4096);
    }

    public HttpResponse performRequest(NetworkRequest request) throws PerformException {
        this.retries408 = 0;

        while(true) {
            BasicHttpResponse response = null;

            try {
                response = this.startHttpConnect(request);
                StatusLine e = response.getStatusLine();
                int statusCode = e.getStatusCode();
                if(statusCode >= 200 && statusCode <= 299) {
                    return response;
                }

                NetworkException networkException = new NetworkException(response);
                if(statusCode != 408 || this.retries408 >= 1) {
                    throw networkException;
                }

                ++this.retries408;
            } catch (SocketTimeoutException var6) {
                attemptRetryOnException(request, new TimeoutException());
            } catch (ConnectTimeoutException var7) {
                attemptRetryOnException(request, new TimeoutException());
            } catch (MalformedURLException var8) {
                throw new RuntimeException("Bad URL " + request.getUrl(), var8);
            } catch (IOException var9) {
                if(response != null) {
                    throw new NetworkException(response, var9);
                }

                throw new NoConnectionException(var9);
            }
        }
    }

    private boolean isHttpProtocol(URL url) {
        return url != null && "http".equals(url.getProtocol());
    }

    private BasicHttpResponse startHttpConnect(NetworkRequest request) throws IOException {
        String url = request.getUrl();
        URL parsedUrl = new URL(url);
        String host = parsedUrl.getHost();
        int port = parsedUrl.getPort();
        boolean hostChanged = false;
        if(NetworkOptions.getUrlConverter() != null && request.isOptimizeDns() && this.isHttpProtocol(parsedUrl)) {
            String connection = NetworkOptions.getUrlConverter().convertUrl(url);
            if(connection == null) {
                throw new IOException("URL blocked by rewriter: " + url);
            }

            parsedUrl = new URL(connection);
            hostChanged = true;
        }

        HttpURLConnection connection1 = this.openConnection(parsedUrl, request);
        Iterator protocolVersion = request.getHeaders().iterator();

        while(protocolVersion.hasNext()) {
            Header responseCode = (Header)protocolVersion.next();
            connection1.addRequestProperty(responseCode.getName(), responseCode.getValue());

        }
        connection1.setRequestProperty("Content-Type", "");
        connection1.setRequestProperty("Host", port < 0?host:host + ":" + port);
        if(hostChanged) {
            connection1.setRequestProperty("FiddlerHost", port < 0?host:host + ":" + port);
            connection1.setRequestProperty("x-online-host", port < 0?host:host + ":" + port);
        }

        setConnectionParametersForRequest(connection1, request);
        ProtocolVersion protocolVersion1 = new ProtocolVersion("HTTP", 1, 1);
        int responseCode1 = connection1.getResponseCode();
        if(responseCode1 < 200 || responseCode1 >= 300) {
            this.printErrorLog(connection1, url, responseCode1 + "");
        }

        if(responseCode1 == -1) {
            throw new IOException("Could not retrieve response code from HttpUrlConnection.");
        } else {
            BasicStatusLine responseStatus = new BasicStatusLine(protocolVersion1, connection1.getResponseCode(), connection1.getResponseMessage());
            BasicHttpResponse response = new BasicHttpResponse(responseStatus);
            response.setEntity(entityFromConnection(connection1));
            Iterator i$ = connection1.getHeaderFields().entrySet().iterator();

            while(i$.hasNext()) {
                Map.Entry header = (Map.Entry)i$.next();
                if(header.getKey() != null) {
                    BasicHeader h = new BasicHeader((String)header.getKey(), (String)((List)header.getValue()).get(0));
                    response.addHeader(h);
                }
            }

            return response;
        }
    }

    private void printErrorLog(HttpURLConnection connection, String oldUrl, String status) {
        if(connection != null) {
            try {
                StringBuffer e = new StringBuffer();
                e.append("status " + status + "\n");
                e.append("原始的url是 " + oldUrl + "\n");
                e.append("connection.getURL()是 " + connection.getURL() + "\n");
                Logger.e("NetworkClientImpl", e.toString());
            } catch (Exception var5) {
                Logger.w("NetworkClientImpl", "打印请求日志信息异常，对业务没有影响" + var5.getMessage());
            }
        }

    }

    private byte[] entityToBytes(HttpEntity entity) throws IOException, ServerException {
        PoolingByteArrayOutputStream bytes = new PoolingByteArrayOutputStream(this.mPool, (int)entity.getContentLength());
        byte[] buffer = null;

        try {
            InputStream in = entity.getContent();
            if(in == null) {
                throw new ServerException();
            } else {
                buffer = this.mPool.getBuf(1024);

                int count;
                while((count = in.read(buffer)) != -1) {
                    bytes.write(buffer, 0, count);
                }

                byte[] var6 = bytes.toByteArray();
                return var6;
            }
        } finally {
            try {
                entity.consumeContent();
            } catch (IOException var13) {
                Log.v("NetworkClient", "Error occured when calling consumingContent");
            }

            this.mPool.returnBuf(buffer);
            bytes.close();
        }
    }

    private static void attemptRetryOnException(NetworkRequest request, PerformException exception) throws PerformException {
        RetryPolicy retryPolicy = request.getRetryPolicy();

        try {
            retryPolicy.retry(exception);
        } catch (PerformException var4) {
            throw var4;
        }
    }

    private static HttpEntity entityFromConnection(HttpURLConnection connection) {
        BasicHttpEntity entity = new BasicHttpEntity();

        InputStream inputStream;
        try {
            inputStream = connection.getInputStream();
        } catch (IOException var4) {
            inputStream = connection.getErrorStream();
        }

        entity.setContent(inputStream);
        entity.setContentLength((long) connection.getContentLength());
        entity.setContentEncoding(connection.getContentEncoding());
        entity.setContentType(connection.getContentType());
        return entity;
    }

    protected HttpURLConnection createConnection(URL url) throws IOException {
        return (HttpURLConnection)url.openConnection();
    }

    private HttpURLConnection openConnection(URL url, NetworkRequest request) throws IOException {
        HttpURLConnection connection = this.createConnection(url);
        connection.setConnectTimeout(request.getTimeoutMs());
        connection.setReadTimeout(request.getReadTimeoutMs());
        connection.setUseCaches(false);
        connection.setDoInput(true);
        if("https".equals(url.getProtocol())) {
            if(this.mSslSocketFactory != null) {
                ((HttpsURLConnection)connection).setSSLSocketFactory(this.mSslSocketFactory);
            }

            if(this.mHostnameVerifier != null) {
                ((HttpsURLConnection)connection).setHostnameVerifier(this.mHostnameVerifier);
            }
        }

        return connection;
    }

    static void setConnectionParametersForRequest(HttpURLConnection connection, NetworkRequest request) throws IOException {
        TypedOutput output = request.getOutput();
        switch(request.getMethod()) {
            case -1:
                if(output.length() > 0L) {
                    connection.setDoOutput(true);
                    connection.setRequestMethod("POST");
                    //connection.addRequestProperty("Content-Type", request.getBodyContentType());
                    connection.setRequestProperty("Content-Type", "");
                    DataOutputStream out = new DataOutputStream(connection.getOutputStream());
                    request.getOutput().writeTo(out);
                    out.close();
                }
                break;
            case 0:
                connection.setRequestMethod("GET");
                break;
            case 1:
                connection.setRequestMethod("POST");
                addBodyIfExists(connection, output);
                break;
            case 2:
                connection.setRequestMethod("PUT");
                addBodyIfExists(connection, output);
                break;
            case 3:
                connection.setRequestMethod("DELETE");
                break;
            case 4:
                connection.setRequestMethod("HEAD");
                break;
            case 5:
                connection.setRequestMethod("OPTIONS");
                break;
            case 6:
                connection.setRequestMethod("TRACE");
                break;
            case 7:
                connection.setRequestMethod("PATCH");
                addBodyIfExists(connection, output);
                break;
            default:
                throw new IllegalStateException("Unknown method type.");
        }

    }

    private static void addBodyIfExists(HttpURLConnection connection, TypedOutput output) throws IOException {
        if(output.length() > 0L) {
            connection.setDoOutput(true);
            //connection.addRequestProperty("Content-Type", output.mimeType());
            connection.setRequestProperty("Content-Type", "");
            DataOutputStream out = new DataOutputStream(connection.getOutputStream());
            output.writeTo(out);
            out.close();
        }

    }

    private static class TypedInputStream implements TypedInput {
        private final String mimeType;
        private final long length;
        private final InputStream stream;

        private TypedInputStream(String mimeType, long length, InputStream stream) {
            this.mimeType = mimeType;
            this.length = length;
            this.stream = stream;
        }

        public String mimeType() {
            return this.mimeType;
        }

        public long length() {
            return this.length;
        }

        public InputStream in() throws IOException {
            return this.stream;
        }
    }

}
