package com.nd.smartcan.content.s3.upload;

import android.util.Log;
import com.nd.android.smartcan.datacollection.EventUtil;
import com.nd.android.smartcan.network.Method;
import com.nd.android.smartcan.network.NetworkClientOkImpl;
import com.nd.android.smartcan.network.NetworkRequest;
import com.nd.android.smartcan.network.exception.PerformException;
import com.nd.android.smartcan.network.mime.TypedFile;
import com.nd.android.smartcan.network.retry.RetryPolicy;
import com.nd.smartcan.commons.util.helper.DateUtil;
import com.nd.smartcan.commons.util.helper.Tools;
import com.nd.smartcan.commons.util.system.SystemInfoUtil;
import com.nd.smartcan.content.s3.utils.S3Util;
import com.nd.smartcan.content.s3.utils.XmlParse;
import com.nd.smartcan.core.restful.ClientResourceUtils;
import com.nd.smartcan.datatransfer.utils.FileUtils;
import com.nd.smartcan.frame.util.AppContextUtils;
import org.apache.http.Header;
import org.apache.http.HttpResponse;
import org.apache.http.message.BasicHeader;
import org.xml.sax.SAXException;
import javax.xml.parsers.ParserConfigurationException;
import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by Administrator on 2016/3/28.
 */
public class UpLoaderImpl implements IUpLoader{

    private static final String TAG = UpLoaderImpl.class.getSimpleName();
    private static final int DEFAULT_MAX_CONNECT_TIME = 10000;
    private static final int DEFAULT_MAX_READ_TIME = 60000;
    public static final int DEFAULT_MAX_NUM_RETRIES = 0;
    private static final String STRING_TYPE = "java.lang.String";
    public static final String UPLOAD_RESULR_SUCCESS = "success";
    private static final SimpleDateFormat LOG_SDF = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss,SSS");
    public static final String SUCCESS = "success";
    public static final String FAILURE = "failure";

    /**
     * Cephs3旁路上传，获取Cephs3上传的token。
     * @param type 文件上传类型 0-一次性上传;1-初始化分块上传;2-分块上传第N块文件;3-完成分块上传
     * @return  json字符串
     */
    @Override
    public String getTokenParams(int type) {
        String result = "";
        switch (type){
            case 0:
                result = "{}";
                break;
            case 1:
                result = "{}";
                break;
            case 2:
                result="{\"uploadId\":1,\"objectName\":1,\"chunkNum\":1}";
                break;
            case 3:
                result="{\"uploadId\":1,\"objectName\":1}";
                break;
        }
        return result;
    }

    @Override
    public String putObject(Map<String, Object> uploadData, File file) {
        String domain = uploadData.get(IUpLoader.UPLOAD_HOST).toString();
        ArrayList<String> ipList = S3Util.getDomainIps(domain);
        String uploadUrl = uploadData.get(IUpLoader.UPLOAD_URL).toString();
        if(ipList != null && ipList.size() > 0){
            for(int i=0; i<ipList.size(); i++){
                try{
                    String requestUrl = uploadUrl.replace(domain,ipList.get(i));
                    HttpResponse response = startUploadRequest(requestUrl, uploadData, file);
                    if(response != null && response.getStatusLine().getStatusCode()==200){
                        return UPLOAD_RESULR_SUCCESS;
                    }
                }catch (Exception e){
                    if(e !=null){
                        try{
                            Log.e(TAG, Log.getStackTraceString(e));
                        }catch(Exception e1){
                        }
                    }
                }
            }
        }else{
            HttpResponse response = startUploadRequest(uploadUrl, uploadData, file);
            if(response != null && response.getStatusLine().getStatusCode()==200){
                return UPLOAD_RESULR_SUCCESS;
            }else{
                return "";
            }
        }
        return "";
    }

    @Override
    public String putObject(Map<String, Object> uploadData, InputStream inputStream, Long length) {
        String localUri = FileUtils.getSDPath().getAbsolutePath() + File.separator + uploadData.get(IUpLoader.UPLOAD_NAME);
        File file = new File(localUri);
        try {
            OutputStream os = new FileOutputStream(file);
            int bytesRead = 0;
            byte[] buffer = new byte[8192];
            while ((bytesRead = inputStream.read(buffer, 0, 8192)) != -1) {
                os.write(buffer, 0, bytesRead);
            }
            os.close();
            inputStream.close();
        } catch (Exception e) {
            if(e !=null){
                try{
                    Log.e(TAG, Log.getStackTraceString(e));
                }catch(Exception e1){
                }
            }
        }

        String domain = uploadData.get(IUpLoader.UPLOAD_HOST).toString();
        ArrayList<String> ipList = S3Util.getDomainIps(domain);
        String uploadUrl = uploadData.get(IUpLoader.UPLOAD_URL).toString();
        if(ipList != null && ipList.size() > 0){
            for(int i=0; i<ipList.size(); i++){
                try{
                    String requestUrl = uploadUrl.replace(domain,ipList.get(i));
                    HttpResponse response = startUploadRequest(requestUrl, uploadData, file);
                    if(response != null && response.getStatusLine().getStatusCode()==200){
                        if(file.exists()){
                            file.delete();
                        }
                        return UPLOAD_RESULR_SUCCESS;
                    }
                }catch (Exception e){
                    if(e !=null){
                        try{
                            Log.e(TAG, Log.getStackTraceString(e));
                        }catch(Exception e1){
                        }
                    }
                }
            }
        }else{
            HttpResponse response = startUploadRequest(uploadUrl, uploadData, file);
            if(response != null && response.getStatusLine().getStatusCode()==200){
                return UPLOAD_RESULR_SUCCESS;
            }else{
                return "";
            }
        }
        return "";
    }

    @Override
    public String initiateMultipartUpload(Map<String, Object> uploadData) {
        String domain = uploadData.get(IUpLoader.UPLOAD_HOST).toString();
        String uploadUrl = uploadData.get(IUpLoader.UPLOAD_URL).toString();
        ArrayList<String> ipList = S3Util.getDomainIps(domain);
        if (ipList != null && ipList.size() > 0) {
            for (int i = 0; i < ipList.size(); i++) {
                try {
                    String requestUrl = uploadUrl.replace(domain, ipList.get(i));
                    HttpResponse response = startInitiateMultipartUploadRequest(requestUrl, uploadData);
                    if (response != null && response.getStatusLine().getStatusCode() == 200) {
                        String result = ClientResourceUtils.readFromInputStream(response);
                        try {
                            String mUploadId = XmlParse.getUploadId(result);
                            return mUploadId;
                        } catch (ParserConfigurationException e) {
                            if(e !=null){
                                try{
                                    Log.e(TAG, Log.getStackTraceString(e));
                                }catch(Exception e1){
                                }
                            }
                        } catch (IOException e) {
                            if(e !=null){
                                try{
                                    Log.e(TAG, Log.getStackTraceString(e));
                                }catch(Exception e1){
                                }
                            }
                        } catch (SAXException e) {
                            if(e !=null){
                                try{
                                    Log.e(TAG, Log.getStackTraceString(e));
                                }catch(Exception e1){
                                }
                            }
                        }
                    }
                } catch (Exception e) {
                    if(e !=null){
                        try{
                            Log.e(TAG, Log.getStackTraceString(e));
                        }catch(Exception e1){
                        }
                    }
                }
            }
        } else {
            HttpResponse response = startInitiateMultipartUploadRequest(uploadUrl, uploadData);
            if (response != null && response.getStatusLine().getStatusCode() == 200) {
                String result = ClientResourceUtils.readFromInputStream(response);
                try {
                    String mUploadId = XmlParse.getUploadId(result);
                    return mUploadId;
                } catch (ParserConfigurationException e) {
                    if(e !=null){
                        try{
                            Log.e(TAG, Log.getStackTraceString(e));
                        }catch(Exception e1){
                        }
                    }
                } catch (IOException e) {
                    if(e !=null){
                        try{
                            Log.e(TAG, Log.getStackTraceString(e));
                        }catch(Exception e1){
                        }
                    }
                } catch (SAXException e) {
                    if(e !=null){
                        try{
                            Log.e(TAG, Log.getStackTraceString(e));
                        }catch(Exception e1){
                        }
                    }
                }
            }
        }
        return "";
    }

    @Override
    public String multipartUploadPartByFile(Map<String, Object> uploadData,String uploadId, File file) {
        String domain = uploadData.get(IUpLoader.UPLOAD_HOST).toString();
        String uploadUrl = uploadData.get(IUpLoader.UPLOAD_URL).toString();
        ArrayList<String> ipList = S3Util.getDomainIps(domain);
        String etag = "";
        if(ipList != null && ipList.size() > 0){
            for(int i=0; i<ipList.size(); i++){
                try{
                    String requestUrl = uploadUrl.replace(domain, ipList.get(i));
                    HttpResponse response = startUploadRequest(requestUrl, uploadData, file);
                    if(response != null && response.getStatusLine().getStatusCode()==200){
                        try{
                            etag = response.getFirstHeader("ETag").getValue();
                            etag = etag.substring(1, etag.length() - 1);
                            return etag;
                        }catch(Exception e){
                            if(e !=null){
                                try{
                                    Log.e(TAG, Log.getStackTraceString(e));
                                }catch(Exception e1){
                                }
                            }
                            return "";
                        }
                    }
                }catch (Exception e){
                    if(e !=null){
                        try{
                            Log.e(TAG, Log.getStackTraceString(e));
                        }catch(Exception e1){
                        }
                    }
                    return "";
                }
            }
        }else{
            HttpResponse response = startUploadRequest(uploadUrl, uploadData, file);
            if(response != null && response.getStatusLine().getStatusCode()==200){
                try{
                    etag = response.getFirstHeader("ETag").getValue();
                    etag = etag.substring(1, etag.length() - 1);
                    return etag;
                }catch(Exception e){
                    return "";
                }
            }else{
                return "";
            }
        }
        return "";
    }

    @Override
    public String multipartUploadPartByStream(Map<String, Object> uploadData, String uploadId, InputStream inputStream, Long length) {
        return null;
    }

    @Override
    public void completeMultipartUpload(Map<String, Object> uploadData, String uploadId, ArrayList<String> etags) throws Exception{
        String domain = uploadData.get(IUpLoader.UPLOAD_HOST).toString();
        String uploadUrl = uploadData.get(IUpLoader.UPLOAD_URL).toString();
        ArrayList<String> ipList = S3Util.getDomainIps(domain);
        if (ipList != null && ipList.size() > 0) {
            for (int i = 0; i < ipList.size(); i++) {
                try {
                    String requestUrl = uploadUrl.replace(domain, ipList.get(i));
                    HttpResponse response = startCompleteMultipartUploadRequest(requestUrl, uploadData, etags);
                    if (response == null || response.getStatusLine().getStatusCode() != 200) {
                        throw new Exception("CompleteMultipartUpload failed");
                    }else{
                        return;
                    }
                } catch (Exception e) {
                    if(i == ipList.size()-1){
                        throw e;
                    }
                }

            }
        } else {
            HttpResponse response = startCompleteMultipartUploadRequest(uploadUrl, uploadData, etags);
            if (response == null || response.getStatusLine().getStatusCode() != 200) {
                throw new Exception("CompleteMultipartUpload failed");
            }else{
                return;
            }
        }
    }

    private HttpResponse startInitiateMultipartUploadRequest(String requestUrl,Map<String, Object> uploadData){
        NetworkClientOkImpl networkClient = new NetworkClientOkImpl();
        NetworkRequest networkRequest = new NetworkRequest();
        networkRequest.timeoutMs(DEFAULT_MAX_CONNECT_TIME);
        networkRequest.readTimeoutMs(DEFAULT_MAX_READ_TIME);
        RetryPolicy policy = networkRequest.getRetryPolicy();
        policy.setMaxNumRetries(0);
        //请求头
        List<Header> headers = new ArrayList<Header>();
        headers.add(new BasicHeader("Content-Type", ""));
        headers.add(new BasicHeader(IUpLoader.UPLOAD_DATE, uploadData.get(IUpLoader.UPLOAD_DATE).toString()));
        headers.add(new BasicHeader(IUpLoader.UPLOAD_AUTHORIZATION, uploadData.get(IUpLoader.UPLOAD_AUTHORIZATION).toString()));
        networkRequest.addHeaders(headers);
        //请求地址
        networkRequest.resetUrl(uploadData.get(IUpLoader.UPLOAD_URL).toString());
        //请求方法
        networkRequest.method(Method.POST);
        //请求消息体 -- 空
        //响应
        HttpResponse response = null;
        try {
            response = networkClient.performRequest(networkRequest);
        } catch (PerformException e) {
            try{
                Log.e(TAG, Log.getStackTraceString(e));
                sendErrorLog(requestUrl,null,networkRequest.getHeaders(),null,0, UpLoaderImpl.FAILURE,Log.getStackTraceString(e));
            }catch(Exception e1){

            }
        }
        return response;
    }

    private HttpResponse startCompleteMultipartUploadRequest(String requestUrl,Map<String, Object> uploadData,ArrayList<String> etags){
        NetworkClientOkImpl networkClient = new NetworkClientOkImpl();
        NetworkRequest networkRequest = new NetworkRequest();
        networkRequest.timeoutMs(DEFAULT_MAX_CONNECT_TIME);
        networkRequest.readTimeoutMs(DEFAULT_MAX_READ_TIME);
        RetryPolicy policy = networkRequest.getRetryPolicy();
        policy.setMaxNumRetries(0);
        //请求头
        List<Header> headers = new ArrayList<Header>();
        headers.add(new BasicHeader(IUpLoader.UPLOAD_DATE, uploadData.get(IUpLoader.UPLOAD_DATE).toString()));
        headers.add(new BasicHeader(IUpLoader.UPLOAD_AUTHORIZATION, uploadData.get(IUpLoader.UPLOAD_AUTHORIZATION).toString()));
        networkRequest.addHeaders(headers);
        //请求地址
        networkRequest.resetUrl(uploadData.get(IUpLoader.UPLOAD_URL).toString());
        //请求方法
        networkRequest.method(Method.POST);
        //请求消息体
        String etagsXml = XmlParse.getCompleteMultipartXmlStream(etags);
        ContentTypedString contentTypedString =  new ContentTypedString(etagsXml); //ContentTypedString类目的是让Http request header 的 Content-type 为空字符串
        networkRequest.output(contentTypedString);
        //响应
        HttpResponse response = null;
        try {
            response = networkClient.performRequest(networkRequest);
        } catch (PerformException e) {
            if(e !=null){
                try{
                    Log.e(TAG, Log.getStackTraceString(e));
                    sendErrorLog(requestUrl,null,networkRequest.getHeaders(),null,0, UpLoaderImpl.FAILURE,Log.getStackTraceString(e));
                }catch(Exception e1){

                }
            }
        }
        return response;
    }

    private HttpResponse startUploadRequest(String requestUrl,Map<String, Object> uploadData,File file){
        S3NetworkClientImpl networkClient = new S3NetworkClientImpl();
        NetworkRequest networkRequest = new NetworkRequest();
        networkRequest.timeoutMs(DEFAULT_MAX_CONNECT_TIME);
        networkRequest.readTimeoutMs(DEFAULT_MAX_READ_TIME);
        RetryPolicy policy = networkRequest.getRetryPolicy();
        policy.setMaxNumRetries(DEFAULT_MAX_NUM_RETRIES);
        //请求头
        List<Header> headers = new ArrayList<Header>();
        headers.add(new BasicHeader("Content-Type", ""));
        headers.add(new BasicHeader(IUpLoader.UPLOAD_DATE, uploadData.get(IUpLoader.UPLOAD_DATE).toString()));
        headers.add(new BasicHeader(IUpLoader.UPLOAD_AUTHORIZATION, uploadData.get(IUpLoader.UPLOAD_AUTHORIZATION).toString()));
        networkRequest.addHeaders(headers);
        //请求地址
        networkRequest.resetUrl(requestUrl);
        //请求方法
        networkRequest.method(Method.PUT);
        //请求消息体
        String mimeType;
        TypedFile typedFile;
        mimeType = ClientResourceUtils.mimeTypeForFileName(file.getName());
        typedFile = new TypedFile(mimeType, file);
        networkRequest.output(typedFile);
        //响应
        HttpResponse response = null;
        try {
            response = networkClient.performRequest(networkRequest);
        } catch (PerformException e) {
            if(e !=null){
                try{
                    Log.e(TAG, Log.getStackTraceString(e));
                    sendErrorLog(requestUrl,null,networkRequest.getHeaders(),null,0, UpLoaderImpl.FAILURE,Log.getStackTraceString(e));
                }catch(Exception e1){

                }
            }
        }
        return response;
    }

    public void sendErrorLog(String url, Object extra, List<Header> requestPropertyHashMap,Map<String, String> responseHashMap,int retry_times,String status,String error){
        Map<String, Object> log = new HashMap<String, Object>();
        log.put("time", DateUtil.getDateString(LOG_SDF));
        log.put("ip", Tools.getHostIp());
        log.put("uid", "");
        log.put("platform", SystemInfoUtil.getModel());
        log.put("app_name", SystemInfoUtil.getAppName(AppContextUtils.getContext()));
        log.put("app_version", SystemInfoUtil.getVersionCode(AppContextUtils.getContext()));
        log.put("elapsed_time", "");

        log.put("deviceId", Tools.getPhoneIMEIorESN(AppContextUtils.getContext()));
        log.put("network", SystemInfoUtil.getNetworkTypeName(AppContextUtils.getContext()));

        log.put("session", "");
        log.put("status", status);
        log.put("retry_times", retry_times);

        Map<String, Object> request = new HashMap<String, Object>();
        request.put("url", url);
        request.put("header", requestPropertyHashMap);
        if(extra !=null){
            request.put("body", String.valueOf(extra));
        }else{
            request.put("body", "");
        }
        log.put("request", request);

        if(responseHashMap !=null){
            log.put("response", responseHashMap);
        }
        log.put("error", error);
        EventUtil.e(AppContextUtils.getContext(), EventUtil.TRACE_NAME, UpLoaderImpl.class.getSimpleName(), log);
    }
}
